package com.ahai.vgk_game_services

import android.app.Activity
import android.content.Intent
import com.google.android.gms.auth.api.Auth
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.games.Games
import com.google.android.gms.drive.Drive
import com.ahai.vgk_game_services.signin.SignInController
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.ActivityResultListener
import java.io.IOException

import java.math.BigInteger
import java.util.Random

private const val CHANNEL_NAME = "vgk_game_services"
private const val RC_SIGN_IN = 9000
private const val SUCCESS = "success";
private const val ERROR = "error";
private const val RESPONSE = "response";
private const val MESSAGE = "message";
private const val ID = "id";
private const val NAME = "name";
private const val DESCRIPTION = "description";
private const val SNAPSHOT_DATA = "snapshotData";
private const val DISPLAY_NAME = "displayName";
private const val SCORE = "score";
private const val PERCENT = "percent";

private object Methods {
  const val signIn = "signIn"
  const val initGame = "initGame"
  const val showLeaderboard = "showLeaderboard"
  const val submitScore = "submitScore"
  const val showAchievements = "showAchievements"
  const val unlockAchievement =  "unlockAchievement"
  const val setPercentAchievement = "setPercentAchievement"
  const val saveSnapShot = "saveSnapShot"
  const val loadSnapShot = "loadSnapShot"
}

class VgkGameServicesPlugin(private var activity: Activity? = null) : FlutterPlugin, MethodCallHandler, ActivityAware, ActivityResultListener {
  private var activityPluginBinding: ActivityPluginBinding? = null
  private lateinit var connectionController: ConnectionController
  private lateinit var signInController: SignInController
  private lateinit var googleSignInClient: GoogleSignInClient
  private lateinit var saveGameName: String
  private lateinit var channel: MethodChannel

  companion object {
    @JvmStatic
    fun registerWith(registrar: PluginRegistry.Registrar) {
      val channel = MethodChannel(registrar.messenger(), CHANNEL_NAME)
      val plugin = VgkGameServicesPlugin(registrar.activity())
      channel.setMethodCallHandler(plugin)
      registrar.addActivityResultListener(plugin)
    }
  }
 
  private fun showLoginErrorIfNotLoggedIn(result: Result) {
    result.success("Please make sure to call signIn() first")
  }

  override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    setupChannel(binding.binaryMessenger)
  }

  override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
  }

  private fun setupChannel(messenger: BinaryMessenger) {
    channel = MethodChannel(messenger, CHANNEL_NAME)
    channel?.setMethodCallHandler(this)
  }

  private fun disposeActivity() {
    activityPluginBinding?.removeActivityResultListener(this)
    activityPluginBinding = null
  }

  override fun onDetachedFromActivity() {
    disposeActivity()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activityPluginBinding = binding
    activity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  private class PendingOperation constructor(val method: String, val result: Result)

  private fun finishPendingOperationWithError(errorMessage: String) {
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    if (requestCode == SignInController.RC_SIGN_IN) {
      val googleSignInResult = Auth.GoogleSignInApi.getSignInResultFromIntent(data)
      signInController.onSignInActivityResult(googleSignInResult)
      return true
    }
    return true
  }

  fun signIn(result: Result) {
    signInController.signIn(googleSignInClient, result)
  }

  fun initGame(result: Result) {
    this.saveGameName =  "SavedGamesName"
    val enableSaveGamesFunctionality = true
    val enablePopups = true
    val activity = this.activity!!
    val signInOptions = if (enableSaveGamesFunctionality) {
        val signInOptionsBuilder = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
        signInOptionsBuilder.requestScopes(Drive.SCOPE_APPFOLDER).requestId()
        signInOptionsBuilder.build()
    } else {
        GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN
    }
    connectionController = ConnectionController(activity, signInOptions)
    signInController = SignInController(activity, connectionController)
    googleSignInClient = GoogleSignIn.getClient(activity, signInOptions)
    signInController.setShowPopups(enablePopups)
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {

      Methods.initGame -> {
        initGame(result)
      }

      Methods.signIn -> {
        signIn(result)
      }

      else -> result.notImplemented()
    }
  }
}