package com.ahai.vgk_game_services.signin

import android.app.Activity
import android.util.Pair
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInResult
import com.google.android.gms.games.Games
import com.ahai.vgk_game_services.ConnectionController
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

class SignInController(
    private var activity: Activity,
    private var connectionController: ConnectionController
) {

    companion object {
        const val RC_SIGN_IN = 77
    }

    private var pendingResult: Result? = null

    private var showPlayPopups = true

    fun setShowPopups(enablePopUps: Boolean) {
        showPlayPopups = enablePopUps
    }

    fun signIn(googleSignInClient: GoogleSignInClient, result: Result) {
        pendingResult = result
        val connection: Pair<Boolean, String> = connectionController.isConnected()
        if (connection.first) {
            result.success("success1-signIn")
            enablePopUps()
        } else {
            googleSignInClient
                .silentSignIn()
                .addOnCompleteListener(activity) { task ->
                    if (task.isSuccessful) {
                        val googleSignInAccount = task.result
                        var accId = ""
                        googleSignInAccount?.id?.let {
                            accId = it
                        }
                        result.success("success-signIn")
                        enablePopUps()
                    } else {
                        val intent = googleSignInClient.signInIntent
                        activity.startActivityForResult(intent, RC_SIGN_IN)
                    }
                }
        }
    }

    fun onSignInActivityResult(googleSignInResult: GoogleSignInResult?) {
        if (googleSignInResult != null && googleSignInResult.isSuccess) {
            val googleSignInAccount = googleSignInResult.signInAccount
            var accId = ""
            googleSignInAccount?.id?.let {
                accId = it
            }
            enablePopUps()
            pendingResult!!.success("success-onSignInActivityResult")
        } else {
            var statusCode = Int.MIN_VALUE
            googleSignInResult?.status?.let {
                statusCode = it.statusCode
            }
            pendingResult!!.success("Failed-onSignInActivityResult")
        }
    }

    fun signOut(googleSignInClient: GoogleSignInClient,  result: Result) {
        googleSignInClient.signOut().addOnCompleteListener(activity) { task ->
            if (task.isSuccessful) {
                result.success("success-signOut")
            } else {
                result.success("failed-signOut")
            }
        }
    }

    fun isSignedIn(): Boolean {
        val googleSignInAccount = GoogleSignIn.getLastSignedInAccount(activity)
        return connectionController.isConnected().first && googleSignInAccount != null
    }

    private fun enablePopUps() {
        if (showPlayPopups) {
            val lastSignedInAccount = GoogleSignIn.getLastSignedInAccount(activity)
            if (lastSignedInAccount != null) {
                Games.getGamesClient(activity, lastSignedInAccount)
                    .setViewForPopups(activity.findViewById(android.R.id.content))
            }
        }
    }
}